/* This file was generated by syncqt. */
#ifndef QT_QTREMOTEOBJECTS_VERSION_H
#define QT_QTREMOTEOBJECTS_VERSION_H

#define QTREMOTEOBJECTS_VERSION_STR "5.9.4"

#define QTREMOTEOBJECTS_VERSION 0x050904

#endif // QT_QTREMOTEOBJECTS_VERSION_H
