/* This file was generated by syncqt. */
#ifndef QT_QTMULTIMEDIAWIDGETS_VERSION_H
#define QT_QTMULTIMEDIAWIDGETS_VERSION_H

#define QTMULTIMEDIAWIDGETS_VERSION_STR "5.9.4"

#define QTMULTIMEDIAWIDGETS_VERSION 0x050904

#endif // QT_QTMULTIMEDIAWIDGETS_VERSION_H
