/* This file was generated by syncqt. */
#ifndef QT_QTFBSUPPORT_VERSION_H
#define QT_QTFBSUPPORT_VERSION_H

#define QTFBSUPPORT_VERSION_STR "5.9.4"

#define QTFBSUPPORT_VERSION 0x050904

#endif // QT_QTFBSUPPORT_VERSION_H
