/* This file was generated by syncqt. */
#ifndef QT_QTSENSORS_VERSION_H
#define QT_QTSENSORS_VERSION_H

#define QTSENSORS_VERSION_STR "5.9.4"

#define QTSENSORS_VERSION 0x050904

#endif // QT_QTSENSORS_VERSION_H
